@@ -60,6 +60,12 @@ sig
   (** Get meaning of Error e *)
   val string_of_error : error -> string
 
+  (** Base port. Default: 80 *)
+  val base_port : int ref
+
+  (** Base host. Default: "post.audioscrobbler.com" *)
+  val base_host : string ref
+
   (** {2 Common API}
 
      Functions common to both basic and advanced APIs *)
@@ -99,8 +105,12 @@ sig
   
   (** [do_np client login song]
      execute a nowplaying request 
-     with authentification *)
-  val do_np : client -> login -> song -> unit
+     with authentification 
+
+     Optional host parameter is a pair
+     "host",port to override the global
+     values.*)
+  val do_np : ?host:(string*int) -> client -> login -> song -> unit
 
   (** [do_submit client login songs]
       execute a nowplaying request 
@@ -110,7 +120,7 @@ sig
      songs for which supplied informations
      were incomplete, with corresponding exception 
      (see [check_song] source) *)
-  val do_submit : client -> login -> song list -> (error * song) list
+  val do_submit : ?host:(string*int) -> client -> login -> song list -> (error * song) list
 
   (** {2 Advanced API} 
      
@@ -131,8 +141,12 @@ sig
     Check do_no and do_submit for examples. *)
 
   (** [handshake client login] 
-     open session, returns session ID *)
-  val handshake : client -> login -> string
+     open session, returns session ID 
+     
+     Optional host parameter is a pair 
+     "host",port to override the global
+     values. *)
+  val handshake : ?host:(string*int) -> client -> login -> string
 
   (** [np sessionID track]
      execute a nowplaying request *)
@@ -170,6 +184,9 @@ sig
   (** Get meaning of Error e *)
   val string_of_error : error -> string
 
+  (** Base host. Default: "ext.last.fm" *)
+  val base_host : string ref 
+
   (** {2 Basic API} *)
   
   (** [get uri] performs whole process and
@@ -210,13 +227,13 @@ sig
 
   (** [parse uri] parse the given lastfm:// uri
     *
-    * returns login option,station,options *)
-  val parse : string -> (login option)*string*(string option)
+    * returns login,station,options *)
+  val parse : string -> login*string*(string option)
   
   (** [init login] initiate lastfm session
     *
     * Returns the session id *)
-  val init : login option -> string
+  val init : login -> string
   
   (** [adjust id station] adjusts lastfm station 
     * for given session ID 
